from typing import TextIO

import click

from cinematinator.instance_io import DebugLetters, parse_cinema, prettyprint_cinema


@click.group()
def cli() -> None:
    pass


@cli.command("print-instance", short_help="Read and prettyprint a Cinema")
@click.argument("input_file", type=click.File(mode="r"))
def print_instance_cmd(input_file: TextIO) -> None:
    """
    Read a Cinema instance from the given file, then pretty-print it with the debug
    character set.

    Pass '-' as the file to read from standard input (which lets you paste in a Cinema).
    """

    cinema = parse_cinema(input_file)
    print(prettyprint_cinema(cinema, DebugLetters))


if __name__ == "__main__":
    cli()
